# Django
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import transaction

class Command(BaseCommand):
    """Load the contents of the entire site from a dumpsite backup."""

    @transaction.commit_on_success
    def handle(self, *args, **options):
        raise NotImplementedError # FIXME



# Python
from optparse import make_option

# Django
from django.conf import settings
from django.core.mail import EmailMessage
from django.core.management.base import BaseCommand
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

# Django-Site-Utils
from site_utils.defaults import SITE_NOTIFY_DEFAULT_RECIPIENTS, \
    SITE_NOTIFY_SUBJECT_TEMPLATE, SITE_NOTIFY_BODY_TEMPLATE
from site_utils.utils import app_is_installed

auth_installed = app_is_installed('django.contrib.auth')
if auth_installed:
    from django.contrib.auth.models import User

class Command(BaseCommand):
    """Management command to notify site admins/managers."""

    option_list = BaseCommand.option_list + (
        make_option('-a', '--admins', action='store_true', dest='admins',
            default=False, help=_('Notify all addresses in settings.ADMINS')),
        make_option('-m', '--managers', action='store_true', dest='managers',
            default=False, help=_('Notify all addresses in settings.MANAGERS')),
    )
    if auth_installed:
        option_list += (
            make_option('-u', '--superusers', action='store_true', dest='superusers',
                default=False, help=_('Notify all users with superuser status')),
            make_option('-s', '--staff', action='store_true', dest='staff',
                default=False, help=_('Notify all users with staff status')),
        )
    option_list += (
        make_option('--bcc', action='store_true', dest='bcc',
            default=False, help=_('BCC all recipients')),
        make_option('--subject-template', action='store', dest='subject_template',
            default=None, help=_('Template to use for email subject.')),
        make_option('--body-template', action='store', dest='body_template',
            default=None, help=_('Template to use for email body.')),
    )
    args = _('[<subject> [<body> ...]]')
    help = _('Send a notification message to site admins/managers.')

    def handle(self, *args, **options):
        verbosity = int(options.get('verbosity', 1))
        admins = bool(options.get('admins', False))
        managers = bool(options.get('managers', False))
        superusers = bool(options.get('superusers', False))
        staff = bool(options.get('staff', False))
        if not (admins or managers or superusers or staff):
            default_recipients = getattr(settings, 'SITE_NOTIFY_DEFAULT_RECIPIENTS',
                                         SITE_NOTIFY_DEFAULT_RECIPIENTS)
            admins = bool('admins' in default_recipients)
            managers = bool('managers' in default_recipients)
            superusers = bool('superusers' in default_recipients and auth_installed)
            staff = bool('staff' in default_recipients and auth_installed)
        bcc = bool(options.get('bcc', False))
        subject_template = getattr(settings, 'SITE_NOTIFY_SUBJECT_TEMPLATE',
                                   SITE_NOTIFY_SUBJECT_TEMPLATE)
        subject_template = options.get('subject_template', None) or subject_template
        body_template = getattr(settings, 'SITE_NOTIFY_BODY_TEMPLATE',
                                SITE_NOTIFY_BODY_TEMPLATE)
        body_template = options.get('body_template', None) or body_template
        subject_text = args[0] if args else _('Site Notification')
        subject_context = {'subject': subject_text}
        subject = render_to_string(subject_template, subject_context)
        subject = ' '.join(filter(None, map(lambda x: x.strip(),
                                            subject.splitlines())))
        subject = settings.EMAIL_SUBJECT_PREFIX + subject
        body_sections = args[1:]
        body_text = '\n\n'.join(body_sections)
        body_context = {'body_sections': body_sections, 'body': body_text}
        body = render_to_string(body_template, body_context)
        recipients = {}
        if admins:
            for name, email in settings.ADMINS:
                recipients.setdefault(email, name)
        if managers:
            for name, email in settings.MANAGERS:
                recipients.setdefault(email, name)
        if superusers:
            for user in User.objects.filter(is_active=True, is_superuser=True):
                recipients.setdefault(user.email, user.get_full_name() or user.email)
        if staff:
            for user in User.objects.filter(is_active=True, is_staff=True):
                recipients.setdefault(user.email, user.get_full_name() or user.email)
        recipient_list = ['"%s" <%s>' % (v,k) for k,v in recipients.items()]
        email_options = {'subject': subject, 'body': body}
        email_options['bcc' if bcc else 'to'] = recipient_list
        email_message = EmailMessage(**email_options)
        email_message.send()



# Python
from optparse import make_option

# Django
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext_lazy as _

# Django-Site-Utils
from site_utils.utils import app_is_installed

class Command(BaseCommand):
    """View/modify the current Site instance in the database."""

    option_list = BaseCommand.option_list + (
        make_option('-n', '--name', action='store', dest='name',
            default=None, help=_('Update the current Site name.')),
        make_option('-d', '--domain', action='store', dest='domain',
            default=None, help=_('Update the Site domain.')),
    )
    help = _('View or modify the current Site in the database.')
    requires_model_validation = True

    def handle(self, *args, **options):
        verbosity = int(options.get('verbosity', 1))
        name = options.get('name', None) or options.get('_name', None)
        domain = options.get('domain', None)
        if not app_is_installed('django.contrib.sites'):
            raise CommandError('Sites app is not installed')
        from django.contrib.sites.models import Site
        site = Site.objects.get_current()
        if name:
            site.name = name
        if domain:
            site.domain = domain
        if name or domain:
            site.save()
        if verbosity >= 1:
            print 'ID=%d Name="%s" Domain="%s"' % (site.pk, site.name, site.domain)



# Python
import os
import sys

# Django
from django.conf import global_settings

# Update this module's local settings from the global settings module.
this_module = sys.modules[__name__]
for setting in dir(global_settings):
    if setting == setting.upper():
        setattr(this_module, setting, getattr(global_settings, setting))

# Absolute path to the directory containing this Django project.
PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    ('Chris Church', 'chris@ninemoreminutes.com'),
    ('David Horton', 'david@ninemoreminutes.com'),
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_ROOT, 'test_project.sqlite3'),
    }
}

SITE_ID = 1

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(PROJECT_ROOT, 'public', 'static')

MIDDLEWARE_CLASSES += (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'devserver.middleware.DevServerMiddleware',
)

TEMPLATE_DIRS = (
    os.path.join(PROJECT_ROOT, 'templates'),
)

ROOT_URLCONF = 'test_project.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'debug_toolbar',
    'devserver',
    'django_extensions',
    'south',
    'site_utils',
    'test_project.test_app',
    'sortedm2m',
    'fortunecookie',
)

INTERNAL_IPS = ('127.0.0.1',)

DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
}

DEVSERVER_DEFAULT_ADDR = '127.0.0.1'
DEVSERVER_DEFAULT_PORT = '8027'

TEST_RUNNER = 'hotrunner.HotRunner'

EXCLUDED_TEST_APPS = [x for x in INSTALLED_APPS \
                      if not x.startswith('test_project.')]

SITE_UPDATE_COMMANDS = {
    'default': [
        'syncdb',
        'migrate',
        'collectstatic',
        'clean_pyc',
    ],
    'other': [
        ('command_does_not_exist', (), {}, 'app_does_not_exist'),
        'other_command_does_not_exist',
    ],
}



# Python
import os

# django-setuptest
import setuptest

class TestSuite(setuptest.setuptest.SetupTestSuite):

    def __init__(self, *args, **kwargs):
        os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'test_settings')
        from south.management.commands import patch_for_test_db_setup
        patch_for_test_db_setup()
        super(TestSuite, self).__init__(*args, **kwargs)

    def resolve_packages(self):
        packages = super(TestSuite, self).resolve_packages()
        if 'test_app' not in packages:
            packages.append('test_app')
        return packages



# Django
from django.core.exceptions import ImproperlyConfigured
from django.db import connection
from django.db.models import get_app

__all__ = ['app_is_installed']

def app_is_installed(app_label):
    """Determine whether the given app is currently installed."""
    try:
        app = get_app(app_label.split('.')[-1])
        return True
    except ImproperlyConfigured:
        return False

def remove_stale_content_types(**options):
    """Remove state content types."""
    verbosity = int(options.get('verbosity', 1))
    dry_run = bool(options.get('dry_run', False))
    if app_is_installed('django.contrib.contenttypes'):
        from django.contrib.contenttypes.models import ContentType
        for ct in ContentType.objects.all():
            if ct.model_class():
                continue
            if verbosity >= 1:
                print 'Removing %r' % ct
            if not dry_run:
                ct.delete()

def remove_unused_database_tables(**options):
    """Remove unused database tables."""
    verbosity = int(options.get('verbosity', 1))
    dry_run = bool(options.get('dry_run', False))
    cursor = connection.cursor()
    all_tables = set(connection.introspection.table_names())
    django_tables = set(connection.introspection.django_table_names())
    for stale_table in (all_tables - django_tables):
        sql = 'DROP TABLE %s;' % stale_table
        if verbosity >= 1:
            print 'Removing table %s' % stale_table
        if verbosity >= 2:
            print sql
        if not dry_run:
            cursor.execute(sql)



# Python
from optparse import make_option

# Django
from django.conf import settings
from django.core.management.base import BaseCommand

# Django-Site-Utils
from site_utils.defaults import SITE_CLEANUP_FUNCTIONS

class Command(BaseCommand):
    """Run various functions to cleanup the site."""

    option_list = BaseCommand.option_list + (
        make_option('--noinput', action='store_false', dest='interactive', default=True,
            help='Tells Django to NOT prompt the user for input of any kind.'),
        make_option('-n', '--dry-run', action='store_true', dest='dry_run',
                    default=False, help='Dry run only.'),
    )
    help = 'Run site cleanup functions.'

    requires_model_validation = True

    def handle(self, *args, **options):
        verbosity = int(options.get('verbosity', 1))
        site_cleanup_functions = []
        for func_spec in getattr(settings, 'SITE_CLEANUP_FUNCTIONS',
                                 SITE_CLEANUP_FUNCTIONS):
            module_name = func_spec.rsplit('.', 1)[0]
            func_name = func_spec.rsplit('.', 1)[1]
            module = __import__(module_name, fromlist=[func_name])
            func = getattr(module, func_name)
            site_cleanup_functions.append(func)
        for func in site_cleanup_functions:
            if verbosity >= 2:
                print 'Running site cleanup function "%s"' % func.__name__
            func(**options)



# Python
from optparse import make_option
import os

# Django
from django.core.management.base import BaseCommand
from django.core.management import call_command, CommandError
from django.conf import settings

# Django-Site-Utils
from site_utils.utils import app_is_installed
from site_utils.defaults import SITE_UPDATE_COMMANDS

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--noinput', action='store_false', dest='interactive', default=True,
            help='Tells Django to NOT prompt the user for input of any kind.'),
    )

    help = 'Execute all management commands for a normal code/database update.'

    requires_model_validation = False

    def handle(self, *args, **options):
        verbosity = int(options.get('verbosity', 1))
        interactive = bool(options.get('interactive', True))
        site_update_commands = getattr(settings, 'SITE_UPDATE_COMMANDS',
                                       SITE_UPDATE_COMMANDS)
        if not args:
            args = ('default',)
        for arg in args:
            if arg not in site_update_commands:
                raise CommandError, 'unknown subcommand %s' % arg

        for arg in args:
            if verbosity >= 2:
                print 'Running site_update commands from group "%s"' % arg
            for cmd_spec in site_update_commands[arg]:
                if isinstance(cmd_spec, basestring):
                    cmd_opts = [cmd_spec, (), {}, None]
                elif isinstance(cmd_spec, (list, tuple)):
                    cmd_opts = (list(cmd_spec) + [None, None, None, None])[:4]
                    if not cmd_opts[0]:
                        raise CommandError, 'no command'
                    cmd_opts[1] = cmd_opts[1] or ()
                    cmd_opts[2] = cmd_opts[2] or {}
                    if isinstance(cmd_opts[3], basestring):
                        cmd_opts[3] = [cmd_opts[3]]
                else:
                    raise CommandError, 'unknown command spec'
                cmd_name = cmd_opts[0]
                cmd_args = cmd_opts[1]
                cmd_options = dict(verbosity=verbosity, interactive=interactive)
                cmd_options.update(cmd_opts[2])
                if not all([app_is_installed(x) for x in cmd_opts[3] or ()]):
                    if verbosity >= 2:
                        print 'Skipping commond "%s"' % cmd_name
                    continue
                if verbosity >= 2:
                    print 'Running command "%s" with args %r and options %r' % (cmd_name, cmd_args, cmd_options)
                call_command(cmd_name, *cmd_args, **cmd_options)



# Python
from optparse import make_option
import os
import shutil
import sys
import tempfile

# Django
from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand, CommandError
from django.core.management.commands.dumpdata import sort_dependencies
from django.core import serializers
from django.db import models, transaction
from django.db import router, DEFAULT_DB_ALIAS
from django.db.models import get_apps, get_models
from django.utils.datastructures import SortedDict

class Command(BaseCommand):
    """Dump the contents of the entire site, including media files."""

    option_list = BaseCommand.option_list + (
        make_option('-o', '--output', default='dumpsite.zip', dest='output',
            help='Specifies the output archive filename.'),
        make_option('--format', default='json', dest='format',
            help='Specifies the output serialization format for fixtures.'),
        make_option('--indent', default=None, dest='indent', type='int',
            help='Specifies the indent level to use when pretty-printing output'),
        make_option('--database', action='store', dest='database',
            default=DEFAULT_DB_ALIAS, help='Nominates a specific database to dump '
                'fixtures from. Defaults to the "default" database.'),
        make_option('-e', '--exclude', dest='exclude',action='append', default=[],
            help='An appname or appname.ModelName to exclude (use multiple --exclude to exclude multiple apps/models).'),
        make_option('-n', '--natural', action='store_true', dest='use_natural_keys', default=True,
            help='Use natural keys if they are available.'),
        make_option('-a', '--all', action='store_true', dest='use_base_manager', default=True,
            help='Use Django\'s base manager to dump all models stored in the database, including those that would otherwise be filtered or modified by a custom manager.'),
    )
    help = ('Dump the contents of the entire site, including media files, to an '
            'archive containing a fixture of the given format.')
    args = '[appname appname.ModelName ...]'

    def handle(self, *app_labels, **options):
        from django.db.models import get_app, get_apps, get_model
        verbosity = int(options.get('verbosity', 1))
        output = options.get('output')
        format = options.get('format')
        indent = options.get('indent')
        using = options.get('database')
        excludes = options.get('exclude')
        show_traceback = options.get('traceback')
        use_natural_keys = options.get('use_natural_keys')
        use_base_manager = options.get('use_base_manager')

        excluded_apps = set()
        excluded_models = set()
        for exclude in excludes:
            if '.' in exclude:
                app_label, model_name = exclude.split('.', 1)
                model_obj = get_model(app_label, model_name)
                if not model_obj:
                    raise CommandError('Unknown model in excludes: %s' % exclude)
                excluded_models.add(model_obj)
            else:
                try:
                    app_obj = get_app(exclude)
                    excluded_apps.add(app_obj)
                except ImproperlyConfigured:
                    raise CommandError('Unknown app in excludes: %s' % exclude)

        if len(app_labels) == 0:
            app_list = SortedDict((app, None) for app in get_apps() if app not in excluded_apps)
        else:
            app_list = SortedDict()
            for label in app_labels:
                try:
                    app_label, model_label = label.split('.')
                    try:
                        app = get_app(app_label)
                    except ImproperlyConfigured:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app in excluded_apps:
                        continue
                    model = get_model(app_label, model_label)
                    if model is None:
                        raise CommandError("Unknown model: %s.%s" % (app_label, model_label))

                    if app in app_list.keys():
                        if app_list[app] and model not in app_list[app]:
                            app_list[app].append(model)
                    else:
                        app_list[app] = [model]
                except ValueError:
                    # This is just an app - no model qualifier
                    app_label = label
                    try:
                        app = get_app(app_label)
                    except ImproperlyConfigured:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app in excluded_apps:
                        continue
                    app_list[app] = None

        # Check that the serialization format exists; this is a shortcut to
        # avoid collating all the objects and _then_ failing.
        if format not in serializers.get_public_serializer_formats():
            raise CommandError("Unknown serialization format: %s" % format)

        try:
            serializers.get_serializer(format)
        except KeyError:
            raise CommandError("Unknown serialization format: %s" % format)

        # Now collate the objects to be serialized.
        objects = []
        for model in sort_dependencies(app_list.items()):
            if model in excluded_models:
                continue
            if not model._meta.proxy and router.allow_syncdb(using, model):
                if use_base_manager:
                    qs = model._base_manager.using(using).all()
                else:
                    qs = model._default_manager.using(using).all()
                objects.extend(qs)
                file_fields = set()
                for field in model._meta.fields:
                    if isinstance(field, models.FileField):
                        print '   ', field.name
                        file_fields.add(field)
                if file_fields:
                    for obj in qs:
                        for field in file_fields:
                            src = getattr(obj, field.name)
                            if src.name:
                                print '   ', obj, field.name, src.name


        
        try:
            serialized_data = serializers.serialize(format, objects, indent=indent,
                        use_natural_keys=use_natural_keys)
        except Exception, e:
            if show_traceback:
                raise
            raise CommandError("Unable to serialize database: %s" % e)

        if 0:
            for app in get_apps():
                for model in get_models(app):
                    print model
                    file_fields = []
                    for field in model._meta.fields:
                        if isinstance(field, models.FileField):
                            print '   ', field.name
                            file_fields.append(field)
                    if file_fields:
                        for obj in model.objects.all():
                            #print obj
                            for field in file_fields:
                                src = getattr(obj, field.name)
                                if src.name:
                                    print '   ', obj, field, src.name
                                if 0 and src:
                                    if not os.path.exists(os.path.dirname(os.path.join(os.path.dirname(__file__), 'backup', src.name))):
                                        os.makedirs(os.path.dirname(os.path.join(os.path.dirname(__file__), 'backup', src.name)))
                                    try:
                                    #if os.path.exists(os.path.join(MEDIA_ROOT, src.path)):
                                        shutil.copy(os.path.join(settings.MEDIA_ROOT, src.path), os.path.join(os.path.dirname(__file__), 'backup', src.name))
                                    except:
                                        print field.name, src



#!/usr/bin/env python

# Python
import sys

# Setuptools
from setuptools import setup, find_packages

# Django-Site-Utils
from site_utils import __version__

setup(
    name='django-site-utils',
    version=__version__,
    author='Nine More Minutes, Inc.',
    author_email='support@ninemoreminutes.com',
    description='Django site-wide management commands and utilities.',
    long_description=file('README', 'rb').read(),
    license='BSD',
    keywords='django site',
    url='https://projects.ninemoreminutes.com/projects/django-site-utils/',
    packages=find_packages(exclude=['test_project']),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django>=1.3',
    ],
    setup_requires=[],
    tests_require=[
        'argparse',
        'Django',
        'django-debug-toolbar',
        'django-devserver',
        'django-extensions',
        'django-fortunecookie',
        #'django-setuptest',
        'django-sortedm2m',
        'South',
    ],
    test_suite='test_suite.TestSuite',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
    ],
    options={
        'egg_info': {
            'tag_svn_revision': 1,
            'tag_build': '.dev',
        },
        'build_sphinx': {
            'source_dir': 'docs',
            'build_dir': 'docs/_build',
            'all_files': True,
            'version': __version__,
            'release': __version__,
        },
        'upload_sphinx': {
            'upload_dir': 'docs/_build/html',
        },
        'aliases': {
            # FIXME: Add 'test' to both aliases below.
            'dev_build': 'egg_info sdist build_sphinx',
            'release_build': 'egg_info -b "" -R sdist build_sphinx',
        },
    },
)



# Python
import sys
import StringIO

# Django
from django.test import TestCase
from django.conf import settings
from django.core import mail
from django.core.management import call_command
from django.contrib.sites.models import Site

# Django-Site-Utils
from site_utils.utils import app_is_installed

class TestSiteUtils(TestCase):
    """Test cases for Site Utils management commands and utilities."""

    def _call_command(self, name, *args, **options):
        original_stdout = sys.stdout
        original_stderr = sys.stderr
        sys.stdout = StringIO.StringIO()
        sys.stderr = StringIO.StringIO()
        result = None
        try:
            result = call_command(name, *args, **options)
        except Exception, e:
            result = e
        finally:
            captured_stdout = sys.stdout.getvalue()
            captured_stderr = sys.stderr.getvalue()
            sys.stdout = original_stdout
            sys.stderr = original_stderr
        return result, captured_stdout, captured_stderr

    def test_app_is_installed(self):
        self.assertTrue(app_is_installed('contenttypes'))
        self.assertTrue(app_is_installed('django.contrib.contenttypes'))
        self.assertTrue(app_is_installed('admin'))
        self.assertTrue(app_is_installed('django.contrib.admin'))
        self.assertFalse(app_is_installed('flatpages'))
        self.assertFalse(app_is_installed('django.contrib.flatpages'))
        self.assertTrue(app_is_installed('site_utils'))

    def test_site_cleanup(self):
        raise NotImplementedError

    def test_site_config(self):
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.com')
        self.assertEqual(site.domain, 'example.com')
        result = self._call_command('site_config', verbosity=0)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.com')
        self.assertEqual(site.domain, 'example.com')
        result = self._call_command('site_config', _name='example.net', verbosity=0)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.net')
        self.assertEqual(site.domain, 'example.com')
        result = self._call_command('site_config', domain='example.net', verbosity=0)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.net')
        self.assertEqual(site.domain, 'example.net')
        result = self._call_command('site_config', _name='example.org',
                                    domain='example.org', verbosity=0)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.org')
        self.assertEqual(site.domain, 'example.org')

    def test_site_dump(self):
        raise NotImplementedError

    def test_site_load(self):
        raise NotImplementedError

    def test_site_notify(self):
        self.assertEqual(len(mail.outbox), 0)
        result = self._call_command('site_notify')
        #print result
        self.assertEqual(len(mail.outbox), 1)
        msg = mail.outbox[-1]
        print msg, msg.to
        raise NotImplementedError

    def test_site_update(self):
        raise NotImplementedError



SITE_CLEANUP_FUNCTIONS = (
    'site_utils.utils.remove_stale_content_types',
    'site_utils.utils.remove_unused_database_tables',
)

SITE_NOTIFY_DEFAULT_RECIPIENTS = ('admins',)

SITE_NOTIFY_SUBJECT_TEMPLATE = 'site_utils/site_notify_subject.txt'

SITE_NOTIFY_BODY_TEMPLATE = 'site_utils/site_notify_body.txt'

SITE_UPDATE_COMMANDS = {
    'default': [
        'syncdb',
        ('migrate', [], {}, 'south'),
        ('collectstatic', [], {}, 'staticfiles'),
    ]
}
